module MqPublisher
  class Publisher
    attr_reader :connection, :channel, :exchange

    def initialize(options={})
      setup_connection(options.slice(:host, :port, :username, :password, :heartbeat, :vhost))
      setup_channel
      setup_exchange(options.slice(:exchange_name, :exchange_type, :exchange_auto_delete, :exchange_durable))
    end

    def publish(message="{}", options={})
      exchange.publish message, options
    end

    def setup_connection(options={})
      @connection ||= Bunny.new(options).tap{ |c| c.start }
    end

    def setup_channel
      @channel ||= connection.create_channel
    end

    def setup_exchange(options={})
      @exchange ||= Bunny::Exchange.new \
        channel,
        options[:exchange_type] || :fanout,
        options[:exchange_name],
        auto_delete: (options[:exchange_auto_delete].nil? ? false : options[:exchange_auto_delete]),
        durable: (options[:exchange_durable].nil? ? false : options[:exchange_durable])
    end

    def destroy
      connection.close
    end
  end
end
